/*
 * FILE --- observer.cc
 *
 * PURPOSE --- Source file for a class that implements a Linear State 
 * Observer for Single-Input-Single-Output (SISO) plants.
 * The user specifies the dimension of the state of the observer and the 
 * observer gains. For the simulation of the dynamics the utility uses a
 * 4'th order Runge-Kutta simulator.
 *
 */

// Include Files
#include "observer.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif

/*************************************************************/

OBSERVER::OBSERVER(int _plant_order, double _sample_time = 0.01)
  // For the observer control inputs are the input and the output 
  // of the actual plant
  : RK4_SIM(_plant_order, 2, _sample_time) 
{
  // No error at start
  observer_err = OBSERVER_NO_ERROR;

  // Allocate memory 
  allocate_memory();
  
  // Check if the RK4 simulator is initialized correctly
  if (rk4_sim_err != RK4_SIM_NO_ERROR)
    {
      observer_err = OBSERVER_MEM_ERROR;
    }

  // If any error then report
  if (observer_err != OBSERVER_NO_ERROR)
    {
#ifdef NO_RCS
      printf("OBSERVER: ERROR --- Not Enought Memory!!!\n");
      printf("The state OBSERVER will not function correctly ...\n");
#else
      rcs_print("OBSERVER: ERROR --- Not Enought Memory!!!\n");
      rcs_print("The state OBSERVER will not function correctly ...\n");
#endif
    }
  
  // Initialize the observer --- onse simulation per cycle
  init_observer(_sample_time, 1);

  // Set all the gains to zero
  set_gains();
}

OBSERVER::~OBSERVER()
{
  // Free the allocated memory
  free_memory();
}

void OBSERVER::allocate_memory(void)
{
  if ((observer_gains = new double[plant_order]) == NULL)
    observer_err = OBSERVER_MEM_ERROR;
}

void OBSERVER::free_memory(void)
{
  delete [] observer_gains;
}

void OBSERVER::init_observer(double _sample_time, 
			     int _sim_per_sample,
			     double _initial_time = 0.0, 
			     double *_initial_state = NULL)
{
  // Set the local parameters
  sample_time = _sample_time;
  sim_per_sample = _sim_per_sample;

  // Initialize the parameters of the integrator, RK4_SIM
  init_param( (sample_time/(double)sim_per_sample), 
	      _initial_time, _initial_state );
}


void OBSERVER::set_gains(double *_observer_gains = NULL)
{
  if (_observer_gains == NULL) // No gains specified
    {
      for (int ii=0; ii < plant_order; ii++)
	{
	  observer_gains[ii] = 0.0;
	}
    }
  else  // Set the specified gains
    {
      for (int ii=0; ii < plant_order; ii++)
	{
	  observer_gains[ii] = _observer_gains[ii];
	}
    }
}


// This function should implement the observer dynamics. Overload this 
// function in the classes that you derive from the OBSERVER class and fill
// it with the dynamics of application specific plants.
void OBSERVER::DERIV(const double t, const double *x, 
		     const double *u, double *xdot)
{
  // Implement your system in the form
  // xhatdot = A*xhat + B*u[0] + L*(u[1] - C*xhat);
  // where u[0] is the control input to the original system and
  // u[1] is the output of the actual system
  // i.e., u[1] = y = C*x

}

