/*
 * FILE --- observer_ex.cc 
 *
 * PURPOSE --- Main source file that provides an example usage of the 
 * usage of the Linear State Observer utility for Single-Input-Single-Output 
 * (SISO) plants. It also provides a simulation of an 
 * example plant whose state is to be observed. In actual implementations 
 * the signal generated from this example will be measured from actual systems.
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "plant_sim.hh"
#include "lqr_ctrl.hh"
#include "my_observer.hh"

void main(void) 
{
  double Ts = 0.01;             // Sampling time
  double t0 = 0.0;              // Initial time 
  double tmax = 10.0;              // Total Simulation period
  int plant_order = 2;  // Order for the plant to be observed  
  double x0[2] = {-20.0, 15.0};  // Initial condistions
  
  
  // State observer gains --- Observer poles are placed at [-1.5, -2.5]
  double observer_gains[2] = {-7.75, 14.75};
  
  // State feedback gains --- System poles are placed at [-1.0, -2.0]
  double feedback_gains[2] = {4.0, 1.0};
  
  // Create the plant
  PLANT_SIM my_plant(Ts, t0, x0);
  
  // Create the controller for the plant
  LQR_CTRL my_controller(plant_order);

  // Create the observer
  MY_OBSERVER my_observer(plant_order, Ts);

  FILE *fp;  // To save data

  if (!(fp = fopen("observer_out.dat","w"))) {
    printf("Can't open file.\n");
    exit(1);
  }

  // Set the gains
  my_controller.set_gains(feedback_gains);
  my_observer.set_gains(observer_gains);

  /* Simulate For tmax seconds. */
  while(my_plant.current_time < tmax) 
    {
      // Calculate the control
      my_plant.ctrl_input[0] = - my_controller.control(my_plant.plant_state);

      // Simulate the plant dynamics
      my_plant.simulate();

      // Simulate the observer dynamics
      my_observer.observe(my_plant.ctrl_input[0], my_plant.plant_output);

      // Save the data
      fprintf(fp,"%f %f %f %f %f %f %f\n", my_plant.current_time,
	      my_plant.plant_state[0], my_plant.plant_state[1],
	      my_plant.plant_output, my_plant.ctrl_input[0],
	      my_observer.plant_state[0], my_observer.plant_state[1]);
    }

  fclose(fp);
}

